"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cli_ux_1 = require("cli-ux");
const chalk_1 = require("chalk");
const abap_1 = require("./lib/abap");
const abap_adt_api_1 = require("abap-adt-api");
const tick = chalk_1.green("\u2713");
const tryLock = async (client, url) => {
    try {
        return await client.lock(url);
    }
    catch (error) {
        if ((error === null || error === void 0 ? void 0 : error.type) === "ExceptionResourceNoAccess" &&
            !(error === null || error === void 0 ? void 0 : error.message.match(/locked/i)))
            return;
        throw error;
    }
};
function validateTransport(lock, key, transport) {
    if (lock.IS_LOCAL && transport)
        throw new Error(`Object ${key} is local, can't use ${transport}`);
    if (!lock.IS_LOCAL && !transport)
        throw new Error(`Object ${key} requires a transport ${lock.CORRNR ? `(locked in ${lock.CORRNR})` : ""}`);
    if (transport !== lock.CORRNR && !lock.IS_LOCAL && lock.CORRNR)
        throw new Error(`Object ${key} locked in transport ${lock.CORRNR} can't use ${transport}`);
}
async function activate(client, incl) {
    if (incl.type === "PROG/I") {
        const main = await client.statelessClone.mainPrograms(incl.metaUrl);
        return client.activate(incl.name, incl.sourceUrl, main === null || main === void 0 ? void 0 : main[0]["adtcore:uri"]);
    }
    const active = await client.activate(incl.name, incl.sourceUrl);
    if (active.inactive.length > 0) {
        const inactives = abap_adt_api_1.inactiveObjectsInResults(active);
        return client.activate(inactives);
    }
    return active;
}
async function write(client, incl, formatted, lock, { test, transport }) {
    var _a;
    const key = `${incl.type} ${incl.name}`;
    if (!lock.LOCK_HANDLE)
        throw new Error(`Failed to lock ${key}`);
    validateTransport(lock, key, transport);
    cli_ux_1.cli.action.start(`\t${key}`, ` Writing...`);
    if (!test)
        await client.setObjectSource(incl.sourceUrl, formatted, lock.LOCK_HANDLE, transport);
    cli_ux_1.cli.action.start(`\t${key}`, ` Unlocking...`);
    await client.unLock(incl.sourceUrl, lock.LOCK_HANDLE);
    cli_ux_1.cli.action.start(`\t${key}`, ` Activating...`);
    if (!test) {
        const active = await activate(client, incl);
        if (active.success === false)
            throw new Error(((_a = active.messages[0]) === null || _a === void 0 ? void 0 : _a.shortText) || `Failed to activate ${key}`);
    }
    cli_ux_1.cli.action.start(`\t${tick}${key}`);
}
async function processInclude(client, incl, options) {
    const key = `${incl.type} ${incl.name}`;
    cli_ux_1.cli.action.start(`\t${key}`, ` Reading...`);
    const source = await client.statelessClone.getObjectSource(incl.sourceUrl);
    cli_ux_1.cli.action.start(`\t${key}`, ` Formatting...`);
    const formatted = await client.statelessClone.prettyPrinter(source);
    if (formatted === source)
        cli_ux_1.cli.action.start(`\t${tick}${key} (unchanged)`);
    else {
        cli_ux_1.cli.action.start(`\t${key}`, ` Locking...`);
        const lock = await tryLock(client, incl.sourceUrl);
        if (lock) {
            await write(client, incl, formatted, lock, options);
        }
        else
            cli_ux_1.cli.action.start(`\t${tick}${key} is generated, skipped`);
    }
    cli_ux_1.cli.action.stop();
}
async function processObjects(client, objects, options) {
    client.stateful = abap_adt_api_1.session_types.stateful;
    try {
        for (const o of objects) {
            cli_ux_1.cli.action.start(`${o.type} ${o.name}`);
            const includes = await abap_1.expand(client, o);
            cli_ux_1.cli.action.start(`${tick}${o.type} ${o.name}`);
            cli_ux_1.cli.action.stop();
            for (const incl of includes)
                await processInclude(client, incl, options);
        }
    }
    catch (error) {
        client.stateful = abap_adt_api_1.session_types.stateless;
        await client.dropSession();
        throw error;
    }
}
exports.processObjects = processObjects;
